// SPDX-License-Identifier: LGPL-3.0-or-later
// See Notices.txt for copyright information
//! ### Destructuring Let
//!
//! You can use `#[hdl] let` to destructure types, similarly to Rust `let` statements with non-trivial patterns.
//!
//! `#[hdl] let` statements can only match one level of struct/tuple pattern for now,
//! e.g. you can match with the pattern `MyStruct { a, b }`, but not `MyStruct { a, b: Struct2 { v } }`.
//!
//! ```
//! # use fayalite::prelude::*;
//! #[hdl]
//! struct MyStruct {
//!     a: UInt<8>,
//!     b: Bool,
//! }
//!
//! #[hdl_module]
//! fn my_module() {
//!     #[hdl]
//!     let my_input: MyStruct = m.input();
//!     #[hdl]
//!     let my_output: UInt<8> = m.input();
//!     #[hdl]
//!     let MyStruct { a, b } = my_input;
//!     #[hdl]
//!     if b {
//!         connect(my_output, a);
//!     } else {
//!         connect(my_output, 0_hdl_u8);
//!     }
//! }
//! ```
