// SPDX-License-Identifier: LGPL-3.0-or-later
// See Notices.txt for copyright information

pub(crate) mod alternating_cell;
mod const_bool;
mod const_cmp;
mod const_usize;
mod misc;
mod scoped_ref;
pub(crate) mod streaming_read_utf8;

// allow easily switching the hasher crate-wide for testing
pub(crate) type DefaultBuildHasher = hashbrown::hash_map::DefaultHashBuilder;

pub(crate) type HashMap<K, V> = hashbrown::HashMap<K, V, DefaultBuildHasher>;
pub(crate) type HashSet<T> = hashbrown::HashSet<T, DefaultBuildHasher>;

#[doc(inline)]
pub use const_bool::{ConstBool, ConstBoolDispatch, ConstBoolDispatchTag, GenericConstBool};
#[doc(inline)]
pub use const_usize::{ConstUsize, GenericConstUsize};

#[doc(inline)]
pub use const_cmp::{
    const_bytes_cmp, const_str_array_is_strictly_ascending, const_str_cmp, const_u8_cmp,
    const_usize_cmp,
};

#[doc(inline)]
pub use scoped_ref::ScopedRef;

#[doc(inline)]
pub use misc::{
    get_many_mut, interned_bit, iter_eq_by, BitSliceWriteWithBase, DebugAsDisplay,
    DebugAsRawString, MakeMutSlice, RcWriter,
};

pub mod job_server;
pub mod prefix_sum;
pub mod ready_valid;
